package org.xbib.groovy.crypt

import java.lang.reflect.Constructor
import java.lang.reflect.InvocationTargetException
import java.security.NoSuchAlgorithmException

/**
 * A utility class that encrypts password strings using algorithms that are
 * compatible with {@code crypt(3)} from the GNU C library.
 */
abstract class Crypt {

    protected final CryptType type

    protected Crypt(CryptType type) {
        this.type = type
    }

    /**
     * Encrypts (digests) the given password using the algorithm identified by the
     * given salt.
     * @param password the password to encrypt
     * @param salt algorithm identifier, parameters, and salt text
     * @return the encrypted (digested) password
     * @throws NoSuchAlgorithmException if the desired algorithm is not supported
     *         on this platform
     * @throws UnsupportedEncodingException if UTF-8 encoding is not available not
     *         the platform
     *
     */
    static String crypt(String password, String salt)
            throws NoSuchAlgorithmException, UnsupportedEncodingException {
        Salt s = new Salt(salt)
        newInstance(CryptType.forSalt(s)).doCrypt(password, s)
    }

    /**
     * Constructs a new instance of the specified type.
     * @param type crypt type
     * @return new crypt object
     */
    private static Crypt newInstance(CryptType type) throws NoSuchAlgorithmException {
        try {
            Constructor<? extends Crypt> constructor = type.providerClass.getConstructor(CryptType)
            constructor.newInstance(type)
        } catch (NoSuchMethodException ex) {
            throw new RuntimeException(ex)
        } catch (IllegalAccessException ex) {
            throw new RuntimeException(ex)
        } catch (InvocationTargetException ex) {
            throw new RuntimeException(ex)
        } catch (InstantiationException ex) {
            throw new RuntimeException(ex)
        }
    }

    /**
     * Converts the encrypted password to a crypt output string.
     * @param password the encrypted password
     * @param salt salt
     * @param maxSaltLength maximum allowable length for the salt
     * @param params subclass-specific parameters (these will be passed to
     * {@link #encodeParameters(Object ...)}
     * @return crypt output string
     */
    protected String passwordToString(byte[] password, Salt salt, int maxSaltLength, Object... params) {
        StringBuilder sb = new StringBuilder()
        if (salt.type == 0) {
            return encodePassword(password)
        }
        sb.append('$').append(salt.type).append('$')
        if (params != null && params.length != 0) {
            String encodedParameters = encodeParameters(params)
            if (encodedParameters != null) {
                sb.append(encodedParameters).append('$')
            }
        }
        sb.append(salt.getText(maxSaltLength)).append('$')
        sb.append(encodePassword(password))
        sb.toString()
    }

    protected abstract String encodeParameters(Object... params)

    /**
     * Performs the password encryption operation.
     * @param password the password to encrypt
     * @param salt salt for the encryption
     * @return formatted crypt output string
     * @throws NoSuchAlgorithmException if the specified encryption type cannot be
     *         supported on the platform
     * @throws UnsupportedEncodingException if the password character encoding
     *         cannot be supported on the platform
     */
    protected abstract String doCrypt(String password, Salt salt)
            throws NoSuchAlgorithmException, UnsupportedEncodingException

    /**
     * Encodes the password for use in the crypt output string.
     * @param password the password to encode
     * @return string encoding of {@code password}
     */
    protected abstract String encodePassword(byte[] password)

}
