package org.xbib.groovy.crypt

import org.xbib.groovy.crypt.random.RandomUtil

import javax.crypto.Mac
import javax.crypto.SecretKeyFactory
import javax.crypto.spec.PBEKeySpec
import javax.crypto.spec.SecretKeySpec
import java.nio.ByteBuffer
import java.nio.ByteOrder
import java.nio.charset.StandardCharsets
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException

/**
 * A utility class for invoking encryption methods and returning LDAP password string,
 * using {@link java.security.MessageDigest} and {@link javax.crypto.Mac}.
 */
class CryptUtil {

    static String hexDigest(String plainText, String algo, String prefix) throws NoSuchAlgorithmException {
        if (plainText == null) {
            return null
        }
        MessageDigest digest = MessageDigest.getInstance(algo)
        digest.update(plainText.getBytes(StandardCharsets.UTF_8))
        "{${prefix}}${digest.digest().encodeHex()}"
    }

    static String base64Digest(String plainText, String algo, String prefix) throws NoSuchAlgorithmException {
        if (plainText == null) {
            return null
        }
        MessageDigest digest = MessageDigest.getInstance(algo)
        digest.update(plainText.getBytes(StandardCharsets.UTF_8))
        "{${prefix}}${digest.digest().encodeBase64()}"
    }

    static String base64Digest(String plainText, byte[] salt, String algo, String prefix) throws NoSuchAlgorithmException {
        if (plainText == null) {
            return null
        }
        MessageDigest digest = MessageDigest.getInstance(algo)
        digest.update(plainText.getBytes(StandardCharsets.UTF_8))
        digest.update(salt)
        byte[] hash = digest.digest()
        byte[] code = new byte[salt.length + hash.length]
        System.arraycopy(hash, 0, code, 0, hash.length)
        System.arraycopy(salt, 0, code, hash.length, salt.length)
        "{${prefix}}${code.encodeBase64()}"
    }

    static String crypt(String plainText, String salt) {
        "{crypt}${Crypt.crypt(plainText, salt)}"
    }

    static String md5(String plainText) {
        base64Digest(plainText, 'MD5', 'md5')
    }

    static String sha(String plainText) {
        base64Digest(plainText, 'SHA', 'sha')
    }

    static String sha256(String plainText) {
        base64Digest(plainText, 'SHA-256', 'sha256')
    }

    static String sha512(String plainText) {
        base64Digest(plainText, 'SHA-512', 'sha512')
    }

    static String ssha(String plainText, byte[] salt) {
        base64Digest(plainText, salt, 'SHA1', 'ssha')
    }

    static String ssha256(String plainText, byte[] salt) {
        base64Digest(plainText, salt, 'SHA-256', 'ssha256')
    }

    static String ssha512(String plainText, byte[] salt) {
        base64Digest(plainText, salt, 'SHA-512', 'ssha512')
    }

    static String pbkdf2(String plainText) {
        // the 389-ds parameters for PBKDF2-SHA256
        pbkdf2(plainText, randomHexString(64), 30000, 256)
    }

    static String pbkdf2(String plainText, String salt) {
        // the 389-ds parameters for PBKDF2-SHA256: 30000 iterations, 64 bytes (128 hex) salt, 256 bytes hash length
        if (salt.length() != 128) {
            throw new IllegalArgumentException("salt must be 64 bytes")
        }
        pbkdf2(plainText, salt, 30000, 256)
    }

    static String pbkdf2(String plainText, String salt, int iterations, int hashLength) {
        byte[] n = htonl(iterations).array() // 4 bytes for network byte order = native byte order
        byte[] b = salt.decodeHex()
        byte[] hash = pbkdf2(plainText.toCharArray(), b, iterations, hashLength * 8)
        int len = n.length + b .length + hash.length
        byte[] result = new byte[len]
        ByteBuffer buffer = ByteBuffer.wrap(result)
        buffer.put(n)
        buffer.put(b)
        buffer.put(hash)
        // 4 + 64 + 256 = 324 bytes
        "{PBKDF2_SHA256}${result.encodeBase64()}"
    }

    static String hmacSHA1(String plainText, String secret) {
        hmac(plainText.getBytes(StandardCharsets.UTF_8), secret.getBytes(StandardCharsets.UTF_8), "HmacSHA1")
    }

    static String hmacSHA1(byte[] plainText, String secret) {
        hmac(plainText, secret.getBytes(StandardCharsets.UTF_8), "HmacSHA1")
    }

    static String hmacSHA1(byte[] plainText, byte[] secret) {
        hmac(plainText, secret, "HmacSHA1")
    }

    static String hmac(String plainText, String secret, String algo) {
        hmac(plainText.getBytes(StandardCharsets.UTF_8), secret.getBytes(StandardCharsets.UTF_8), algo)
    }

    static String hmac(byte[] plainText, String secret, String algo) {
        hmac(plainText, secret.getBytes(StandardCharsets.UTF_8), algo)
    }

    static String hmac(byte[] plainText, byte[] secret, String algo) throws NoSuchAlgorithmException {
        Mac mac = Mac.getInstance(algo)
        mac.init(new SecretKeySpec(secret, algo))
        mac.doFinal(plainText).encodeHex()
    }

    /**
     *  Computes the PBKDF2 hash of a password.
     *
     * @param   password    the password to hash.
     * @param   salt        the salt
     * @param   iterations  the iteration count (slowness factor)
     * @param   bytes       the length of the hash to compute in bytes
     * @return              the PBDKF2 hash of the password
     */
    static byte[] pbkdf2(char[] plainText, byte[] salt, int iterations, int len) {
        PBEKeySpec spec = new PBEKeySpec(plainText, salt, iterations, len)
        SecretKeyFactory skf = SecretKeyFactory.getInstance("PBKDF2WithHmacSHA256")
        skf.generateSecret(spec).getEncoded()
    }

    static String randomHexString(int length) {
        RandomUtil.randomBytes(length).encodeHex()
    }

    static ByteBuffer htonl(int value) {
        ByteBuffer.allocate(4).order(ByteOrder.nativeOrder()).putInt(value)
    }
}
