package org.xbib.groovy.crypt.random

/**
 * RNG seed strategy that gets data from {@literal /dev/random} on systems
 * that provide it (e.g. Solaris/Linux).  If {@literal /dev/random} does not
 * exist or is not accessible, a {@link SeedException} is thrown.
 */
class DevRandomSeedGenerator implements SeedGenerator {

    private static final File DEV_RANDOM = new File("/dev/random")

    /**
     * Generate seed.
     * @return The requested number of random bytes, read directly from
     * {@literal /dev/random}.
     * @throws SeedException If {@literal /dev/random} does not exist or is
     * not accessible
     */
    @Override
    byte[] generateSeed(int length) throws SeedException {
        FileInputStream file = null
        try {
            file = new FileInputStream(DEV_RANDOM)
            byte[] randomSeed = new byte[length]
            int count = 0
            while (count < length) {
                int bytesRead = file.read(randomSeed, count, length - count)
                if (bytesRead == -1) {
                    throw new SeedException("end-of-file while reading random data")
                }
                count += bytesRead
            }
            return randomSeed
        }
        catch (IOException ex) {
            throw new SeedException("failed reading from " + DEV_RANDOM.getName(), ex)
        }
        catch (SecurityException ex) {
            throw new SeedException("security prevented access to " + DEV_RANDOM.getName(), ex)
        }
        finally {
            if (file != null) {
                try {
                    file.close()
                }
                catch (IOException ex) {
                    // ignore
                }
            }
        }
    }

}